<%--
 ** Component: cases
 **
 ** Performs study operations initiated by a remote dotcom server.
 **
 ** @param study  Study Instance UID of study to perform operation on
 ** @param oper   Operation to be performed on the study
 **
 ** $Author$
 ** $Date$
 **
 *
 *  RCS File Name    : $RCSfile$
 *  RCS Revision     : $Revision$
 *  Locked by        : $Locker$
 *  Symbolic name    : $Name$
 *  State            : $State$
--%><%@
	page import="imagemed.util.*, java.io.*, imagemed.imsession.*, imagemed.jsputil.*, java.util.*, imagemed.taskd.IMScriptContext,
			imagemed.dotcom.*, imagemed.dcprocessing.StudyRemover, imagemed.exception.IMException"

%><%!

	static Syslog logger = new Syslog( "changestudyprp", 0, IMEnvBase.getEnv( "LOG_FACILITY" ) );

	// For disabling setting the same study paralelly
	static LockMgr m_lockManager = new LockMgr();
	
	/**
	 * Creates a system lock on a study for passing/receiving ownership
	 *
	 * @param is     session stuff
	 * @param study  Study Instance UID of the study
	 *
	 * @return lockID
	 */
	String createSystemLock( IMSession is, String study ) {
		IMLock lock = IMLock.createLocalStudyLock( study, is );
		return lock.lock( 100 );
	}


	/**
	 * Release a system lock from a study
	 *
	 * @param is     session stuff
	 * @param study   Study Instance UID of the study
	 * @param lockID  Lock ID generated by createSystemLock()
	 */
	void removeSystemLock( IMSession is, String study, String lockID ) {
		IMLock lock = IMLock.createLocalStudyLock( study, is, lockID );
		lock.unlock();
	}


	/**
	 * Starts setstudyprp.sh tasks by putting them to the NET queue.
	 *
	 * @param context  Task context received from peer
	 * @param study    Study Instance UID of the study
	 * @param oper     Operation to be performed on the remote dotcom device
	 * @param device   ID of the remote dotcom device to perform operation on
	 * @param bUnlock  Is there any lock created on the study, which should be released
	 */
	void runRetryTask( IMScriptContext context, String study, String oper, String device, boolean bUnlock ) {
		StringBuffer cmd = new StringBuffer( 256 );
		cmd.append( Helper.convertPathForJavaTask( IMEnvBase.getEnv( "CASES_HOME" ) ) ).append( ".SetStudyPrp" );
		if ( bUnlock )
			cmd.append( " -u" );
		cmd.append( " -s \"" ).append( study ). append( "\" -o " ). append( oper ). append( " -H " ).append( device );
		context.createChildTask( "SET_STUDY_PROPERTY", cmd.toString(), "class" );
	}

	/**
	 * Sets one of the staged devices as study owner
	 * (pass the study ownership to one of the staged children)
	 *
	 * @param is               Session stuff - required for releasing the lock
	 * @param context  Task context received from peer
	 * @param study            Study Instance UID
	 * @param storestate       Content of the storestate.rec file of the study
	 * @param lockID           Lock ID of system lock to be released after taking ownership
	 *
	 * @throws Throwable if an error occurs
	 */
	void setChildOwner( IMSession is, IMScriptContext context, String study, StoreState storestate, String lockID ) throws Throwable {
		Set<String> stagedDevs = storestate.getStagedDevices();
		
		String newOwnerDevice = null;
		for ( Iterator<String> i = stagedDevs.iterator(); i.hasNext(); ) {
			String dev = i.next();
			if ( DotcomConfig.getConfig().getDevice( dev ).isDbOnly() )
				continue;
			newOwnerDevice = dev;
			break;
		}

		if ( newOwnerDevice == null ) {
			// We can't get ownership from parent since we are 'Deleted' (or 'Database' ???)
			// and the study is not on any child server
			// Take ownership as if we can inherit so Deleted chain remains consistent
			Vector<String> modStorestate = new Vector<String>();
			modStorestate.add( "--owner" );
			modStorestate.add( "yes" );
			storestate.modify( modStorestate );
			removeSystemLock( is, study, lockID );
			return;
		}

		Vector<String> modStorestate = new Vector<String>();
		modStorestate.add( "--setowner" );
		modStorestate.add( newOwnerDevice );

		runRetryTask( context, study, "setowner", newOwnerDevice, true );

		storestate.modify( modStorestate );
	}


	/**
	 * Inherits the study ownership from a child or parent. If the ownership could not be inherited,
	 * the ownership will be passed to toward the archive sarver or to the parent server (if any).
	 *
	 * @param is               session stuff - required for releasing the lock
	 * @param context  Task context received from peer
	 * @param study            Study Instance UID
	 * @param storestate       Content of the storestate.rec file of the study
	 * @param lockID           Lock ID of system lock to be released after taking ownership
	 *
	 * @throw Throwable if an error occurs
	 */
	void inheritOwnership( IMSession is, IMScriptContext context, String study, StoreState storestate, String lockID ) throws Throwable {

		String processMode = storestate.getAttribute( "ProcessMode" );
		String dotcomDevices[] = storestate.getDotcomDevices();
		String stagedDevices = storestate.getAttribute( "StagedDevices" );
		if ( "".equals( stagedDevices ) || "NULL".equals( stagedDevices ) )
			stagedDevices = null;

		// check if we can inherit ownership
		if ( "Database".equals( processMode ) || "Deleted".equals( processMode ) ) {
			// can't inherit ownership, pass it

			DotcomDevice selfDevice = DotcomConfig.selfDevice();
			String parentDevice = storestate.getNextToStudyTop( selfDevice );

			if ( parentDevice != null ) {
				boolean hasOtherStagedDevices = false; // has other devices than parent
				for ( int i = 0; i < dotcomDevices.length; i++ ) {
					if ( !parentDevice.equals( dotcomDevices[ i ] ) ) {
						hasOtherStagedDevices = true;
						break;
					}
				}
				
				runRetryTask( context, study, hasOtherStagedDevices ? "setstaged" : "removesender", parentDevice, true );
			}
			else { // (parentDevice == null)
				if ( null == stagedDevices ) {
					// we can get here only if there are neither OwnerDevice nor StagedDevices

					// release lock
					removeSystemLock( is, study, lockID );
				}
				else {
					setChildOwner( is, context, study, storestate, lockID );
				}
			}
		}
		else { // ! ("Database".equals(processMode) || "Deleted".equals(processMode) )
			// Ownership inherited
			// release lock
			removeSystemLock( is, study, lockID );
		}
	}


%><%-- Input Parameters --%><%

	IMSession is = new IMSession( request, response, out );
	String parStudy = request.getParameter( "study" );  // study   - Study IUID
	String parOper = request.getParameter( "oper" );    // operation - {removereference, removesender, setowner, setstaged}

	//Syslog s = new Syslog("changestudyprp", 0, IMEnvBase.getEnv("LOG_FACILITY"));
	String errMsg = null;

	// Set regID for creating system lock
	is.setRegID( "#SYSTEM#" );

	if ( parStudy == null || parOper == null ) {
%>false<%
		return;
	}

	logger.syslog( Syslog.LOG_DEBUG,parOper + " " + parStudy );

	IMScriptContext context = IMScriptContext.readJobContextFrom( request );
	
	String lockID = null;

	m_lockManager.lock( parStudy );
	try {

		StoreState storestate = new StoreState( parStudy );
		storestate.load();

		/********************************
		*
		*  Set study ownership
		*
		********************************/
		if ( "setowner".equals( parOper ) ) {
			// setowner request may come from either child or parent
			Properties remotePrp = DotcomToolbox.checkClient( is, null, false );

			String ownerDevice = storestate.getAttribute( "OwnerDevice", "NULL" );

			if ( ownerDevice.equals( remotePrp.getProperty( "ID" ) ) ) {
				// create system lock
				lockID = createSystemLock( is, parStudy );
				if ( lockID == null ) {
					throw new IMException( "The study " + parStudy + " is locked" );
				}

				Vector<String> modStorestate = new Vector<String>();

				String processMode = storestate.getAttribute( "ProcessMode", "Deleted" );
				if ( "Deleted".equals( processMode ) || "Database".equals( processMode ) ) {
					// setowner on one of the children (or parent - since multiple archive support)
					setChildOwner( is, context, parStudy, storestate, lockID );
				}
				else {
					storestate.setMode( StoreState.EXISTING_ONLY );
					modStorestate.add( "--owner" );
					modStorestate.add( "yes" );
					modStorestate.add( "--setstaged" );
					modStorestate.add( ownerDevice );
					storestate.modify( modStorestate );
					// release lock
					removeSystemLock( is, parStudy, lockID );
					lockID = null;
				}
			}
		}

		/********************************
		*
		*  Add child to StagedIPs
		*
		********************************/
		else if ( "setstaged".equals( parOper ) ) {
			// setstaged request may come from either parent or child since multiple archive support 
			// was introduced, but always comes from owner device
			Properties remotePrp = DotcomToolbox.checkClient( is, null, false );

			String ownerDevice   = storestate.getAttribute( "OwnerDevice", "NULL" );

			// If the request was received from non Owner device then do nothing
			if ( ownerDevice.equals( remotePrp.getProperty( "ID" ) ) ) {
				// create system lock
				lockID = createSystemLock( is, parStudy );
				if ( lockID == null ) {
					throw new IMException( "The study " + parStudy + " is locked" );
				}

				// set senderip to null, set ownership, and add ip to stagedips
				Vector<String> modStorestate = new Vector<String>();
				modStorestate.add( "--setstaged" );
				modStorestate.add( ownerDevice );
				modStorestate.add( "--owner" );
				modStorestate.add( "yes" );

				storestate.setMode( StoreState.EXISTING_ONLY );
				storestate.modify( modStorestate );

				inheritOwnership( is, context, parStudy, storestate, lockID );
				lockID = null;
			}
		}

		/********************************
		*
		*  Remove reference
		*
		********************************/
		else if ( "removereference".equals( parOper ) ) {
			// removereference request may com from either parent or child since multiple archive support was introduced
			Properties remotePrp = DotcomToolbox.checkClient( is, null, false );
			String remoteDevice = remotePrp.getProperty( "ID", "NULL" );

			Vector<String> modStorestate = new Vector<String>();
			modStorestate.add( "--removedevice" );
			modStorestate.add( remoteDevice );

			storestate.setMode( StoreState.EXISTING_ONLY );
			storestate.modify( modStorestate );

			// Remove child_<device> reference from STUDYDIR
			ReferenceCounter.removeReference( "STUDYDIR", parStudy, "child_" + remoteDevice );
		}

		/********************************
		*
		*  Remove sender // from 4.3: Remove Owner
		*
		********************************/
		else if ( "removesender".equals( parOper ) || "removeowner".equals( parOper ) ) {
			// removeowner request may come from either parent or child since multiple archive support
			// was introduced, but always comes from owner device
			Properties remotePrp = DotcomToolbox.checkClient( is, null, false );
			String remoteDevice = remotePrp.getProperty( "ID", "NULL" );

			String ownerDevice  = storestate.getAttribute( "OwnerDevice" );

			// Remove OwnerDevice if it's remoteDevice
			if ( ownerDevice != null && ownerDevice.equals( remoteDevice ) ) {
				Vector<String> modStorestate = new Vector<String>();
				modStorestate.add( "--removedevice" );
				modStorestate.add( ownerDevice );
				modStorestate.add( "--owner" );
				modStorestate.add( "yes" );

				// lock study
				lockID = createSystemLock( is, parStudy );
				if ( lockID == null ) {
					throw new IMException( "The study " + parStudy + " is locked" );
				}

				storestate.setMode( StoreState.EXISTING_ONLY );
				storestate.modify( modStorestate );

				// Remove child_<device> reference from STUDYDIR
				ReferenceCounter.removeReference( "STUDYDIR", parStudy, "child_" + remoteDevice );

				// check if we can inherit the study
				inheritOwnership( is, context, parStudy, storestate, lockID );
				lockID = null;
			}
		}

		/********************************
		 *
		 *  Nuke study
		 *
		 ********************************/
		else if ( "nuke".equals( parOper ) ) {
			Properties remotePrp = DotcomToolbox.checkClient( is, null, false );
			String remoteDevice = remotePrp.getProperty( "ID" );
			
			// In case of nuking orders indicates that the order is corrected to a study
			// and attachments scanned for the order should not be automatiacally deleted 
			boolean isCompleted = "true".equals( request.getParameter( "completed" ) );

			if ( ! StudyToolbox.studyExists( parStudy ) ) {
				// The study does not exist - return a special value to let the child_ reference be removed from the peer (CBT #373)
				logger.syslog( Syslog.LOG_INFO, "Nuke study " + parStudy + " [study did not exist]" );
				response.setContentType( "text/plain" );
				response.setStatus( 200 );
				out.print( "nosuchstudy" );
				return;
			}
			
			// call nukestudy
			// XXX: when passing task context between peers will be implemented, task context should also be passed to nukeStudy
			if ( !StudyRemover.nukeStudy( null, parStudy, remoteDevice, null, isCompleted ) ) {
				StringBuilder cmd = new StringBuilder( 256 );
				cmd.append( Helper.convertPathForJavaTask( IMEnvBase.getEnv( "CASES_HOME" ) ) )
						.append( ".NukeStudy -s \"" ).append( parStudy )
						.append( "\" -H \"" ).append( remoteDevice).append( "\"" );
				if ( isCompleted ) {
					cmd.append( " -c" );
				}
				context.createChildTask( "NUKE_STUDY", cmd.toString(), "class" );
			}
			logger.syslog( Syslog.LOG_INFO,"Nuke study " + parStudy );
		}

		/*************************************
		*
		*  Pass ownership to (Master) parent
		*
		************************************/
		else if ( "passownership".equals( parOper ) ) {
			// passownership request may come from either parent or child since multiple archive support was introduced
			Properties peerPrp = DotcomToolbox.checkClient( is, null, false );
			String peerId = peerPrp.getProperty( "ID" );
			if ( peerId == null )
				throw new IMException( "Peer ID is null" );
			
			String studyDir = RepositoryHandler.getLocation( RepositoryHandler.DATA, parStudy );
			String legacyRefcountDir = IMEnvBase.getEnv( "REFCOUNT_VAR" ) + "/OBJECTFILE/" + parStudy;
			String refcountDir = IMEnvBase.getEnv( "REFCOUNT_VAR" ) + "/OBJECTFILE/" + DirHash.getHashed( parStudy );
			
			//check if no autofwd and fwd reference on study and the ownership can be passed
			String cmd[] = new String[ 7 ]; 
			//String env[] = new String[1];

			cmd[ 0 ] = "/bin/grep";
			cmd[ 1 ] = "-r";
			cmd[ 2 ] = "-e";
			cmd[ 3 ] = "autofwd";
			cmd[ 4 ] = "-e";
			cmd[ 5 ] = "forward_" + peerId;
			cmd[ 6 ] = refcountDir;
			
			RunCommandResult res = Helper.runCommandEx( cmd, null );
			if ( res.getExitCode() == 2 ) {
				// If executing grepo fails on regular reference counters (directory do not exist), then
				// try running on legacy reference counters
				cmd[ 6 ] = legacyRefcountDir;
				res = Helper.runCommandEx( cmd, null );
			}
			
			if ( res.getExitCode() == 0 ) {
				throw new IMException( "Study ownership cannot be passed, there are objects still being forwarded to " + peerId + " for " + parStudy );
			}
			else if ( res.getExitCode() == 1 && ! "".equals( res.getOutput() ) ) {
				throw new IMException( "Study ownership cannot be passed, there are objects still being forwarded to " + peerId + " for " + parStudy);
			}
			else if ( res.getExitCode() == 2 ) {
				if ( studyDir != null && ( new File( refcountDir ).exists() || new File( legacyRefcountDir ).exists() ) )
					throw new IMException( "Error occured during checking reference autoforward and forward on study: " + parStudy );
			}

			if ( studyDir != null ) {
				//pass ownership
				String studyOwner = storestate.getAttribute( "StudyOwner" );
				if ( "yes".equals( studyOwner ) ) {
					// create system lock
					lockID = createSystemLock( is, parStudy );
					if ( lockID == null ) {
						throw new IMException( "The study " + parStudy + " is locked" );
					}
					Vector<String> modStorestate = new Vector<String>();
					if (! "NULL".equals( storestate.getAttribute( "OwnerDevice", "NULL" ) ) ) {
						modStorestate.add( "--removedevice" );
						modStorestate.add( storestate.getAttribute( "OwnerDevice" ) );
					}
	
					modStorestate.add( "--setowner" );
					modStorestate.add( peerId );
	
					storestate.modify( modStorestate );
	
					// release lock
					removeSystemLock( is, parStudy, lockID );
					lockID = null;
				}
				else {
					String ownerDevice = storestate.getAttribute( "OwnerDevice" );
					 
					if	( !peerId.equals( ownerDevice ) ) {
						DotcomDevice selfDevice = DotcomConfig.selfDevice();
						if ( selfDevice.isParent( ownerDevice ) || selfDevice.isChild( ownerDevice ) ) {						
							String taskcmd = Helper.convertPathForJavaTask( IMEnvBase.getEnv("CASES_HOME") ) + ".StealOwnership -s \"" + parStudy + "\"";
							context.createChildTask( "STEAL_STUDY_OWNERSHIP", taskcmd, "class", "DELTA_STEAL_STUDY_OWNERSHIP_PROXY" );
						}
						errMsg="changestudyprp.jsp : Stealing ownership from " + ownerDevice;
						logger.syslog( Syslog.LOG_DEBUG, errMsg );
					}
				}
			}
		}

		/********************************
		*
		*  Obsolete study
		*
		********************************/
		else if ( "obsolete".equals( parOper ) ) {
			Properties remotePrp = DotcomToolbox.checkClient( is, null, false );
			String remoteDevice = remotePrp.getProperty( "ID", "NULL" );

			// call obsolete
			// XXX: when passing task context between peers will be implemented, task context should also be passed to obsoleteStudy
			if ( !StudyRemover.obsoleteStudy( null, parStudy, remoteDevice, null ) ) {
				String cmd = Helper.convertPathForJavaTask( IMEnvBase.getEnv( "CASES_HOME" ) ) + ".ObsoleteStudy -s \"" + parStudy + "\" -H \"" + remoteDevice + "\"";
				context.createChildTask( "OBSOLETE_STUDY", cmd, "class" );
			}
			logger.syslog( Syslog.LOG_INFO, "Obsolete study " + parStudy );
		}
		
		/********************************
		*
		*  Invalidate study by date/time
		*
		********************************/
		else if ( "invalidate".equals( parOper )  ) {
			Properties remotePrp = DotcomToolbox.checkClient( is, null, false );
			String remoteDevice = remotePrp.getProperty( "ID", "NULL" );
			String parTimeStamp = request.getParameter( "timestamp" );

			// call obsolete
			// XXX: when passing task context between peers will be implemented, task context should also be passed to obsoleteStudy
			if ( !StudyRemover.invalidate( null, parStudy, parTimeStamp, remoteDevice, null ) ) {
				
				String cmd = Helper.convertPathForJavaTask( IMEnvBase.getEnv( "CASES_HOME" ) ) + ".ObsoleteStudy -s \"" +
						parStudy + "\" -H \"" + remoteDevice + "\"" + " -t \"" + parTimeStamp + "\"";
				context.createChildTask( "OBSOLETE_STUDY", cmd, "class" );
			}
			
			logger.syslog( Syslog.LOG_INFO, "Invalidate study " + parStudy + " [" + parTimeStamp + "]");
		}

		/********************************
		*
		*  Set Parent staged
		*
		********************************/
		else if ( "setparentstaged".equals( parOper ) ) {
			Properties peer = DotcomToolbox.checkClient( is, "C", false );

			storestate.setMode( StoreState.CREATE_ONLY );
			Vector<String> modStorestate = new Vector<String>();

			modStorestate.add( "--processmode" );
			modStorestate.add( IMEnv.getConfigProperty( IMEnvBase.getEnv( "CONF" ) + "/self.rec", "ProcessMode", "Process" ) );
			modStorestate.add( "--setowner" );
			modStorestate.add( peer.getProperty( "ID" ) );

			storestate.modify( modStorestate );
			
			DotcomToolbox.setParentStaged( null, parStudy );
		}

		/****** Invalid operation ******/
		else {
			throw new Throwable( "Invalid operation" );
		}
	}
	catch ( Throwable th ) {
		errMsg = th.getMessage() == null ? "EXCEPTION: " + Helper.toString(th) : th.getMessage();
		logger.syslog( Syslog.LOG_ERR, errMsg );

		if ( lockID != null )
			removeSystemLock( is, parStudy, lockID );
	}
	finally {
		m_lockManager.release( parStudy );
	}

	response.setContentType( "text/plain" );


%><%= errMsg == null ? "true" : "false" %><%

	if ( true ) return; // jsp puts an extra '\n' if we don't return
%>
