<%--
 * Component: cases
 *
 * Description:
 *  Interface for locking/unlocking studies used by the viewer and remote dotcom servers.
 *
 * Parameters:
 *
 *   cStudy        Study Instance UID
 *   cOp           Lock operation: Lock, Break, CheckLock, IsLocked, Unlock (= default), lockOwners, None
 *   cClose        true - indicates the study is being closed in the viewer
 *                    in case of cClose=true only Unlock and None are the valid lock operations
 *
 *
--%><%@
	page import="imagemed.util.*, imagemed.imsession.*, imagemed.jsputil.*, java.util.*, java.sql.*,
			imagemed.types.flag.IMTypeur"

%><%@
	page errorPage="../main/exceptionHandler.jsp"

%><%

	IMSession is = new IMSession( request, response, out ); 
	String parOp = request.getParameter( "cOp" );
	String parStudy = request.getParameter( "cStudy" );
	boolean isClose = "true".equals( request.getParameter( "cClose" ) );
	// multiple studies may be presented to operation "lockOwners" only
	String[] parStudies = request.getParameterValues( "cStudy" );

	if ( parOp == null )
		parOp = "Unlock";

	Syslog logger = new Syslog( "lockstudy.jsp", 0, IMEnvBase.getEnv( "LOG_FACILITY" ) );
	if ( !"lockOwners".equals( parOp ) ) { // lockOwners is inner call between servers, should not be logged.
		logger.syslog( Syslog.LOG_DEBUG, parStudy + " " + is.COOKIE.getProperty( "sessionid" ) );
	}
	if ( null == parStudy ) {
		logger.syslog( Syslog.LOG_DEBUG, "Study is null" );
		response.setHeader( "IMLock", "FAILED" );
%>FAILED<%return;
	}

	is.bCheckHttpsAndRedirect = false;
	if ( ! ProxyToolbox.init( is, null, logger ) && ! is.regOK() ) {
		if ( "Unlock".equals( parOp ) ) {
			// Allow releasing locks for anyone - fail otherwise
			is.regRights = "R";
			if ( is.regID == null && is.prevUserID != null ) {
				// Pick up last valid user from session if any - for logging 
				is.setRegID( is.prevUserID );
				is.regGroup = "";
			}
		}
		else {
			// Fail otherwise
			logger.syslog( Syslog.LOG_DEBUG, "regOK failed" );
			return;
		}
	}
	
	// reg.jspi sets is.regID to null
	if ( is.regID == null )
		is.setRegID( request.getHeader( "IMRegID" ) );
	if ( is.regID == null )
		is.setRegID( request.getParameter( "cRegID" ) );

	if ( request.getHeader( "IMLockID" ) == null && request.getParameter( "cIMLockID" ) != null ) {
		if ( is.sessionVariables == null )
			is.sessionVariables = new Properties();
		is.sessionVariables.setProperty( "IMLockID", request.getParameter( "cIMLockID" ) );
	}

	if ( isClose ) {
		OperLogEntry logEntry = new OperLogEntry( OperLogEntry.LEVEL.STUDY, "close" );
		logEntry.setField( OperLogEntry.FIELD.USER, is.regID );
		logEntry.setField( OperLogEntry.FIELD.STUDY, parStudy );

		try ( ResultSet rs = IMEnv.getDB().prepareStatement( "select * from Dcstudy WHERE STYIUID='" + parStudy + "'" ).executeQuery() ) {
			if ( rs.next() ) {
				logEntry.setField( OperLogEntry.FIELD.PNAME, rs.getString( "PNAME" ) );
				logEntry.setField( OperLogEntry.FIELD.MODALITY, rs.getString( "MODALITY" ) );
				logEntry.setField( OperLogEntry.FIELD.PID, rs.getString( "PID" ) );
				logEntry.setField( OperLogEntry.FIELD.ACCNO, rs.getString( "ACCNO" ) );
				logEntry.setField( OperLogEntry.FIELD.STUDYDATE, rs.getString( "THEDATETIME" ) );
				logEntry.setField( OperLogEntry.FIELD.INSTITUTION, rs.getString( "INSTNAME" ) );
				logEntry.setField( OperLogEntry.FIELD.STUDYDESCR, rs.getString( "STYDESCR" ) );
			}
		}
		catch ( SQLException sqlEx ) {
			logger.syslog( Syslog.LOG_ERR, Helper.throwable2String( sqlEx ) );
		}
		logEntry.log( "lockstudy", 0, Syslog.LOG_INFO );
	}

	if ( null == is.regRights || ( -1 == is.regRights.indexOf( "R" ) &&  -1 == is.regRights.indexOf( "G" ) && -1 == is.regRights.indexOf( "T" ) ) ) {
		logger.syslog( Syslog.LOG_DEBUG, "No Rights" );
		response.setHeader( "IMAuthentication", "norights" );%>FAILED<%
		return;
	}

	if ( "Break".equals( parOp ) && !is.hasRight( IMTypeur.BREAK_LOCK_RIGHT ) ) { 
		logger.syslog( Syslog.LOG_DEBUG, "No Break Rights" );
		response.setHeader( "IMAuthentication", "norights" );
%>FAILED<%
		return;
	}

	if ( "lockOwners".equals( parOp ) ) { // lockOwners is inner call between servers, should not be logged.
		Set< String > studySet = new HashSet <String >();
		studySet.addAll( Arrays.asList( parStudies ) );
		LockStatManager lsm = new LockStatManager( is, studySet );
		for ( int i = 0; i < parStudies.length; i++ ) {
			String locker = lsm.getLockOwner( parStudies[ i ] );
			if ( locker != null ) {
				response.setHeader( parStudies[ i ], locker );
			}
		}
%>OK<%
		return;	
	}

	// in case of shortcuts the original study should be locked
	String sourceStudy = StudyToolbox.getSourceStudyDB( parStudy );

	if ( ProxyToolbox.checkAndProxy( is, sourceStudy, false ) ) {
		logger.syslog( Syslog.LOG_DEBUG, "checkAndProxy is true" );
		return;
	}

	Properties rspHeaders = new Properties();
	IMLock imLock = IMLock.createLocalStudyLock( parStudy, is );
	String res = imLock.localLock( parOp, rspHeaders );

	for ( Enumeration<?> e = rspHeaders.propertyNames(); e.hasMoreElements(); ) {
		String name = ( String ) e.nextElement();
		response.setHeader( name, rspHeaders.getProperty( name ) );
	}

	if ( "OK".equals( res ) ) {
%>OK<%
		return;
	}
	logger.syslog( Syslog.LOG_DEBUG, "failed: res=" + res );

%>FAILED<%

	if ( true )
		return;

%>
