<%--
* Component: cases
*
* Description:
*  Interface for uploading new DICOM files by the viewer.
*
* Parameters (multipart):
*
*   cStudy        Study Instance UID
*   cSOP          SOP Instance UID of the new DICOM object
*   cModality     Modality
*   cFile         The content of the new DICOM object
*
* Last Updated     : $Author$
* Update Date Time : $Date$
* RCS File Name    : $RCSfile$
* RCS Revision     : $Revision$
* Locked by        : $Locker$
* Symbolic name    : $Name$
* State            : $State$
*
--%><%@
	page import="imagemed.util.*,imagemed.imsession.*,imagemed.jsputil.*,java.io.*,imagemed.types.flag.IMTypeur,java.util.*"

%><%@
	page errorPage ="../main/exceptionHandler.jsp"

%><%!

	static String CASES;
	static {
		String cases = IMEnvBase.getEnv("CASES_HOME");
		int pos = cases.lastIndexOf('/');
		if (pos != -1) {
			cases = cases.substring(pos + 1);
		}
		CASES = cases;
	}


	boolean putDicomFile( String tmpFileName, String viewSessionID, String study, Properties filePrp ) {
		String[] cmd = new String[ 2 ];
		
		cmd[ 0 ] = IMEnvBase.getEnv( "SR_HOME" ) + "/bin/saveUploaded";
		cmd[ 1 ] = tmpFileName;
		
		Helper.DEBUG( "putDicomFile", cmd[ 0 ] + " " + cmd[ 1 ] );
		
		RunCommandResult res = Helper.runCommandEx( cmd, null );
		if ( res.getExitCode() == 0 ) {
			// parse returned data into an ObjectFingerPrint and add it to the viewer's 
			try {
				filePrp.load( new StringReader( res.getOutput() ) );
				
				File fVidFile = new File( IMEnvBase.getEnv( "LOGIN_TMP" ) + "/" + viewSessionID + ".vid" );
				if ( viewSessionID != null && fVidFile.exists() ) {
					String sopiuid = filePrp.getProperty( "SOPInstanceUID" );
					String sopcluid = filePrp.getProperty( "SOPClassUID" );
					boolean isImage = "true".equals( filePrp.getProperty( "IsImage" ) );
					String date = filePrp.getProperty( "PBAcquisitionDate" );
					String time = filePrp.getProperty( "PBAcquisitionTime" );
					VIDManager.rmiAddToVidFile( null, viewSessionID, study, sopiuid, sopcluid, isImage, date, time );
				}
			}
			catch ( Throwable th ) {
				Helper.defaultCatch( th );
			}
			return true;
		}
		return false;
	}


	void createObjectLogEntry( Syslog logger, String study, String object, String modality, String user ) {
		logger.syslog( Syslog.LOG_INFO, "INFO OBJECT upload " + user + " " + object + " " +
			study + " - " + modality + " - - - - - - - - - -" );
	}


%><%
	IMSession is = new IMSession(request, response, out);

	MultipartRequest mp = new MultipartRequest(is, IMEnvBase.getEnv("TMP"));

	String parStudy    = mp.getParameter("cStudy");
	String parSOP      = mp.getParameter("cSOP");
	String parModality = mp.getParameter("cModality");
	String parFile     = mp.getParameter("cFile");

	Syslog logger = new Syslog("putDicomFile", 0, IMEnvBase.getEnv("LOG_FACILITY"));
	logger.syslog(Syslog.LOG_DEBUG, parStudy + " " + parSOP + " " + parFile + " " + parModality + " " +
			is.COOKIE.getProperty("sessionid"));

	if (!mp.isMultiPart()) {
		logger.syslog(Syslog.LOG_DEBUG,"Not Multipart request");
		%>FAILED<%
		return;
	}



/*
	// Uncomment this section to log parameters in debug log

	for (Enumeration e = mp.getParameterNames(); e.hasMoreElements();) {
		String parName = (String) e.nextElement();
		String parVals[] = mp.getParameterValues(parName);
		if (parVals == null) continue;
		for (int i = 0 ; i < parVals.length ; i++)
			logger.syslog(Syslog.LOG_DEBUG,"PARAM " + parName + " = [" + parVals[i] + "]");
	}
*/

	is.bCheckHttpsAndRedirect = false;
	if ( ! ProxyToolbox.init( is, mp, logger ) ) {
		if ( !is.regOK() ) {
			return;
		}
	}

	if (null == parStudy || null == parSOP || null == parFile || null == parModality) {
		logger.syslog(Syslog.LOG_DEBUG,"Missing parameter(s)");
		%>FAILED<%
		return;
	}


%><%--	// TODO: Checking upload Rights instead of report right ???? --%><%
	if (!(is.hasRight(IMTypeur.EDIT_RIGHT) || is.hasRight(IMTypeur.REPORT_RIGHT))) {
		logger.syslog(Syslog.LOG_DEBUG,"No Rights");
		response.setHeader("IMAuthentication","norights");
		%>FAILED<%return;
	}


	if ( ProxyToolbox.checkAndProxy(is, parStudy, mp, false) ) {
		logger.syslog(Syslog.LOG_DEBUG,"checkAndProxy is true");

		if (!is.bRegNotNeeded) {
			createObjectLogEntry(logger, parStudy, parModality + "." + parSOP, parModality, is.regID);
		}

		return;
	}

	Properties filePrp = new Properties();
	if ( putDicomFile( parFile, mp.getHeader( "ViewSessionID" ), parStudy, filePrp ) ) {
		createObjectLogEntry( logger, parStudy, parModality + "." + parSOP, parModality, is.regID );
		OperLogEntry logEntry = new OperLogEntry( OperLogEntry.LEVEL.STUDY, "uploadFromViewer" );
		logEntry.setBasicStudyFields( is, parStudy );
		StringBuilder logInfo = new StringBuilder( 256 );
		String fileName = filePrp.getProperty( "FileName" );
		String sopcluid = filePrp.getProperty( "SOPClassUID" );
		if ( fileName != null ) {
			logEntry.setField( OperLogEntry.FIELD.FNAME, fileName );
			logInfo.append( ";FileName:" ).append( fileName );
		}
		if ( sopcluid != null ) {
			logEntry.setField( OperLogEntry.FIELD.CHANGES, sopcluid );
			logInfo.append( ";SOPClass:" ).append( sopcluid );
		}
		if ( logInfo.length() > 1 ) {
			logEntry.setField( OperLogEntry.FIELD.MESSAGE, logInfo.substring( 1 ) );
		}
		if ( is.regID != null ) {
			logEntry.setField( OperLogEntry.FIELD.USER, is.regID );
		}
		logEntry.log( new Syslog( "putDicomFile.jsp", 0, IMEnvBase.getEnv( "LOG_OPER_FACILITY" ) ), Syslog.LOG_INFO );
		new File( parFile ).delete();
		
		%>OK<%
	}
	else {
		new File( parFile ).delete();
/*		for (Enumeration e = rspHeaders.propertyNames() ; e.hasMoreElements() ; ) {
			String name = (String) e.nextElement();
			response.setHeader(name,rspHeaders.getProperty(name));
		}*/
		%>FAILED<%
		return;
	}

	if (true)
		return;
%>
