<%--
 ** Component: cases
 **
 ** Description:
 ** 	Splits study into two or more studies
 **
 ** Parameters:
 **
 **	@param cBack        URL to go back after splitting the study
 **	@param cStudy       Study Instance UID of the study to be split
 **	@param cOp          Split operation (Save | Clear | Confirm | Split)
 **	@param S<N>         Series Instance UID of the Nth series ( 0 <= N < number of series in the study)
 **	@param O<N>         File name of the Nth image object ( 0 <= N < number of image objects in the study)
 **
 ** Operations:
 **
 ** 	Save: Stores split settings in temporary files per new study.
 **		The content of a temporary file is:
 **			accno   : accession number of the new study
 **         reports : true indicates to copy the report objects to the study
 **			mode    : split on series (0) or objects (1)
 **			from    : serial number of the first series/object to be copied from the original study
 **			to      : serial number of the last series/object to be copied from the original study
 **			uids    : list if series uids/object file names to be copied
 **
 **	Clear: Clears split settings by removing saved temporary files.
 **
 **	Confirm: Display confirmation page before do the actual splitting
 **
 **	Split: Do the actual work
 **
 ** On the split editor and confirmation pages the content will always be displayed from the settings
 ** stored in the temporary files.
 **
 ** $Author$
 ** $Date$
 **
 *
 *  RCS File Name    : $RCSfile$
 *  RCS Revision     : $Revision$
 *  Locked by        : $Locker$
 *  Symbolic name    : $Name$
 *  State            : $State$
--%><%@
	page import="java.util.*, imagemed.imsession.*, imagemed.util.*, imagemed.jsputil.*,
		imagemed.exception.*, java.text.Format,
		imagemed.tables.*, imagemed.imtables.*, imagemed.types.string.*"

%><%@
	page errorPage ="../main/exceptionHandler.jsp"

%><%

	Splitter splitter = new Splitter(request, "cases/index.jsp");

	IMSession is = new IMSession(request, response, out);

	is.response.setHeader("pragma","no-cache");
	is.response.setHeader("Cache-Control" , "no-cache, must-revalidate");
	is.response.setHeader("Expiry-Date" , "Friday, 31-Dec-70 23:59:59 GMT");

	if ( !is.regOK() ) {
		return;
	}

%><%-- Some setup --%><%// pageSize and refresh comes from defs.jspi

	String defaultFields[] = { "DATE" , "PANM", "STAT" , "MODY", "DSCR", "PHRF", "PHRD", "NIMG" };
	String SQLtables[]     = { "Dcstudy", "Dcobject" };

	String msgSplitError = LocalizedMessage.getSimpleMessage( "CASES_SPLIT_ERROR" );
	
	boolean isAdmin         = (is.regRights != null && is.regRights.indexOf("A") != -1);
	boolean hasEditRight    = (is.regRights != null && is.regRights.indexOf("E") != -1);

	if (!isAdmin && !hasEditRight)
		throw new IMCommonGUIException( is, LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_SPLIT_NO_RIGHTS" ), splitter.getTab(), msgSplitError, splitter.getBack(), 0 );

	if (splitter.getStudy() == null)
		throw new IMCommonGUIException( is, LocalizedMessage.getSimpleMessage( "GENERAL_MISSING_PARAMETER" ), splitter.getTab(), msgSplitError, splitter.getBack(), 0 );

	// Redirect from CWL to hub
	String redirHost = RedirectToolbox.getRedirectForStudy( is, splitter.getStudy() );
	if ( redirHost != null ) {
		String redirURL = redirHost + "/cases/split.jsp?" + RedirectToolbox.getRedirectParams( is );
		response.sendRedirect( redirURL );
		return;
	}

	try {
		EditToolbox.isStudyEditable( is, splitter.getStudy(), "", "" );
	}
	catch ( IMGeneralException e ) {
		throw new IMCommonGUIException( is, LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_SPLIT_READ_ONLY" ), splitter.getTab(), msgSplitError, splitter.getBack(), 0 );
	}

	
	TableDescriptor desc = new TableDescriptor();

	FieldManager fm = new IMSQLFieldManager("Dcfields", "Dctypes", SQLtables);
	desc.setFieldManager(fm);

	
	
	String sourceStudy = StudyToolbox.getSourceStudyDB( splitter.getStudy() );
	if ( !splitter.getStudy().equals( sourceStudy ) ) {
		// Shortcut to be split, check if the user has access to the source study
		
		Properties filters = new Properties();
		filters.setProperty( "SYUI", sourceStudy );
	
		WorklistTableParams tparams = new WorklistTableParams( is.imRequest, is.getUser() );
		tparams.setDefaultFields(defaultFields);
		tparams.setDefaultFilters(filters);
		desc.setTableParams(tparams);
	
		WorklistResult wr = new WorklistResult(is, desc);
	
		if ( !wr.next() ) {
			throw new IMCommonGUIException( is, LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_SPLIT_SHORTCUT" ), splitter.getTab(), msgSplitError, splitter.getBack(), 0 );
		}
		
		MessageBox mb = new MessageBox( LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_SPLIT_SPLITTING_SHORTCUT" ), "60%" );
		mb.setMessage( "<H3><BR>" + LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_SPLIT_SHORTCUT" ) + "<BR>" +
		 		LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_SPLIT_SHORTCUT_CLICK" ) + "<BR><BR></H3>" );

		StringBuilder url = new StringBuilder( 256 );
		url.append( "/cases/split.jsp?cStudy=" );
		url.append( Helper.URLEncodeUTF8( sourceStudy ) );
		if ( splitter.getBack() != null ) {
			url.append( "&cBack=" ).append( Helper.URLEncodeUTF8( splitter.getBack() ) );
		}
		if ( splitter.isIface() ) {
			url.append( "&cIface=" ).append( splitter.getIfaceMode() );
		}
		
		if ( splitter.getBack() != null ) {
			mb.addButton( new Button( splitter.getBack(), "BUTTON_CANCEL", "BUTTON_CANCEL" ) );
		}
		throw new IMMessageBoxException( is, mb, splitter.getTab() ); // , 0, splitter.getBack() );
	}
	
	
	Properties filters = new Properties();
	filters.setProperty("SYUI", splitter.getStudy());

	WorklistTableParams tparams = new WorklistTableParams( is.imRequest, is.getUser() );
	tparams.setDefaultFields(defaultFields);
	tparams.setDefaultFilters(filters);
	desc.setTableParams(tparams);

	WorklistResult wr = new WorklistResult(is, desc);
	wr.setGlobal( true );

	if (!wr.next())
		throw new IMCommonGUIException( is, LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_NO_SUCH_STUDY" ), splitter.getTab(), msgSplitError, splitter.getBack(), 0 );

%><%-- ----- HTML code ----- --%><HTML>
<HEAD><TITLE><%=is.http_server%> - <%=LocalizedMessage.getSimpleMessage( "CASES_SPLIT_STUDY" )%></TITLE>
 <SCRIPT Language="JavaScript">
    function FullWindow_( pStudy_, pFname_, pI_, pWindow_, pX_, pY_ )
    { vX = parseInt( pX_ ) + 30;        //2 pix + opt. scroll
      if ( vX < 200 )
        vX = 200;                       //for text width in bottom frame
      else if ( 800 < vX )
        vX = 800;
      vY = parseInt( pY_ ) + 54;        //fot text height in bottom frame
      if ( 600 < vY )
        vY = 600;
      w = window.open( 'qvwin.jsp?cStudy='+pStudy_+'&cFile=' +pFname_+ '&cI=' +pI_+ '&cX=' +pX_+ '&cY=' +pY_ , pWindow_,
                   'menubar=no,location=no,resizable=yes,scrollbars=yes,status=no,width=' + vX + ',height=' + vY );
  	  w.focus();

    }

    function doSubmit(frm,val)
    {
      frm.cOp.value = val;
      frm.submit();
    }
 </script>
	<%
		Table.printCommonScript(is);
	%>
</HEAD>
<BODY>
<%
	if ( splitter.getTab() != null ) {
		TabToolbox.makeTabs( is,is.tabbean, splitter.getTab(), "White");
	}

	TableProperties tprop = new TableProperties();
	String colStudyRowShadow = tprop.getColor("StudyRowShadow");

	// get patient name to be printed in the header
	String pname = wr.getValue("PANM");
	IMTypePANM displayPANM = new IMTypePANM();
	pname = displayPANM.display(pname);


	// Process Request Parameters
	try{
	splitter.save(is.out);

	splitter.load();
	splitter.checkSplit();

	// Start splitting if necessary
	if (splitter.doSplit(is)) {
		out.println("<BR><CENTER>");
		MessageBox mb = new MessageBox(LocalizedMessage.getSimpleMessage( "CASES_SPLIT_STARTED" ), "60%");
		if ( splitter.getBack() != null ) {
			mb.addButton(new Button(splitter.getBack(), "BUTTON_BACK_TO_LIST", "ALT_TEXT_BACK"));
		}
		Format[] msgFormat = new Format[] { null };
		Object[] msgObject = new Object[] { pname };
		String msg = LocalizedMessage.getCompoundMessage( "CASES_SPLIT_STUDY_OF_PNAME_STARTED", msgFormat, msgObject, true);  
		mb.setMessage("<CENTER><h3>" + msg + "</h3></CENTER>");
		mb.print(is.out);
		out.println("</center></body></html>");
		return;
	}


	Properties prp = splitter.isStudyLocked(is);
	if (prp != null) {
		String lockerName = prp.getProperty("IMLockOwner");
		String lockAge    = prp.getProperty("IMLockAge");
		lockerName = (null == lockerName) ? LocalizedMessage.getSimpleMessage( "GENERIC_UNKNOWN_USER" ) : IMTypePN.displayOrdinaryText(lockerName);
		lockAge = ReportToolbox.getLockAgeMessage(lockAge);
		if (lockAge.length() > 4)
			lockAge = " " + lockAge.substring(4).trim() + "";
		Format[] msgFormat = new Format[] { null, null };
		Object[] msgObject = new Object[] { lockerName, lockAge };
		
		String msg = LocalizedMessage.getCompoundMessage( "STUDY_LOCKED", msgFormat, msgObject, true);
		throw new IMCommonGUIException( is, "<center>" + LocalizedMessage.getSimpleMessage( "EXCEPTION_CASES_CANT_SPLIT_BECAUSE" ) + "</center><BR>" + msg,
				splitter.getTab(), msgSplitError, splitter.getBack(), 0 );
	}


	splitter.printJSFunction(is);
%>
<h3 style="text-align: center;"><%= LocalizedMessage.getCompoundMessage( "CASES_SPLIT_STUDY_OF_PNAME", new Format[] { null }, new Object[] { pname }, true ) %></h3>

<FORM name="splitform" action="split.jsp" method="post">
<INPUT TYPE=HIDDEN NAME=cStudy VALUE="<%= Helper.htmlEncode(splitter.getStudy()) %>">
<INPUT TYPE=HIDDEN NAME=cOp>
<%

	String refererHost = request.getParameter( "RefererHost" );
	if ( refererHost != null ) {
%>
<INPUT type="hidden" name="RefererHost" VALUE="<%=Helper.htmlEncode( refererHost ) %>">
<%
	}
	if ( splitter.getBack() != null ) {
%>
<INPUT TYPE=HIDDEN NAME=cBack VALUE="<%= Helper.htmlEncode( splitter.getBack() ) %>">
<%
	}
	
	if ( splitter.isIface() ) {
%>
<INPUT TYPE=HIDDEN NAME=cIface VALUE="<%= splitter.getIfaceMode() %>">
<%
	
	}

	splitter.printHeaderTable(is, "BgShadow");
	splitter.display(is.out);
	}catch( Exception e ){
		// error message and operation buttons showed in MessageBox
		String title = LocalizedMessage.getSimpleMessage( "CASES_SPLIT_ERROR" );
		String message = LocalizedMessage.getSimpleMessage("CASES_SPLIT_EXCEPTION_OCCURS");
		Button[] buttons = splitter.getButtonsIfNecessary();
		out.println(splitter.getPrintableMessageBox( title, message, buttons ));
		String throwableString = Helper.throwable2String( e );
		Helper.ERRORLOG( "split.jsp", throwableString);
		out.println( "<!--" + throwableString + "-->" );
	}
%>
</FORM>
</BODY></HTML>
